// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: fifo_cmd_xactor
// PURPOSE: Transactor for the fifo
class Fifo_cmd_xactor extends vmm_xactor;
  import fifo_pkg::*;
  virtual fifo_if.fdrvr_if_mp f_if;
  Fifo_xactn_channel    in_chan; // input channnel for transactions 
  Fifo_response_channel resp_chan;  // output responsse channel
  function new(string inst,
	           int unsigned stream_id = -1,
               virtual fifo_if.fdrvr_if_mp  new_vir_if,
               Fifo_xactn_channel           new_in_chan=null,
               Fifo_response_channel        fifo_response_channel=null);
      super.new("cmd_xactor", inst, stream_id);
      this.f_if = new_vir_if;
      if (new_in_chan==null)
 		 this.in_chan=new("fifo_chan","channel");
      else this.in_chan = new_in_chan;
      if (fifo_response_channel==null)
        this.resp_chan=new("fifo_response_chan", "channel");
      else this.resp_chan=fifo_response_channel;
  endfunction : new

  extern task main();
  extern task reset_task (int num);
  extern task push_task (word_t data);
  extern task pop_task();
  extern task push_pop_task (word_t data);
  extern task idle_task (int num_idle_cycles);
  
endclass: Fifo_cmd_xactor
  
  task Fifo_cmd_xactor::main();
    Fifo_xactn  fifo_xactn_0;  // transaction to get 
    Fifo_response  fifo_response; // response to generator
    fork
      super.main();
    join_none
    forever
    begin : main_loop
      // Rule 4-121
      this.in_chan.peek(fifo_xactn_0);
      // Rule 4-123
      fifo_xactn_0.notify.indicate(vmm_data::STARTED);    

      `vmm_trace(log, $psprintf("Got a new fifo xaction from in_channel %s ",								   
                                  fifo_xactn_0.psdisplay()));
      case (fifo_xactn_0.kind)
        PUSH :
        begin
          this.push_task(fifo_xactn_0.data);
        end
        POP  :
        begin
         this.pop_task(); 
        end
        PUSH_POP :
        begin
         this.push_pop_task(fifo_xactn_0.data);
        end
        IDLE :
        begin
          this.idle_task(fifo_xactn_0.idle_cycles);
        end
        RESET :
          this.reset_task(fifo_xactn_0.reset_cycles); 
      endcase
      // Can do checks here if needed
      fifo_response=new();
      fifo_response.kind = fifo_xactn_0.kind;
	  fifo_response.status= PASSED;
      // .. 
      // Now do a get() to unblock producer
      // Rule 4-123
      fifo_xactn_0.notify.indicate(vmm_data::ENDED, fifo_response);
      // Send the response to generator thru the response class object. 
      // in nonblocking manner.
      this.resp_chan.sneak(fifo_response);
      // Rule 4-121      
      this.in_chan.get(fifo_xactn_0);    
    end : main_loop
  endtask : main

  task Fifo_cmd_xactor::reset_task (int num);
    @ ( f_if.driver_cb);
    for (int i=0; i < num; i++) 
    begin 
      `TOP.reset_n <= 1'b0;  
      idle_task(num); 
    end
    `TOP.reset_n <= 1'b1;
    @ ( f_if.driver_cb);
    @ ( f_if.driver_cb);	   
  endtask : reset_task
     
  task Fifo_cmd_xactor::push_task (word_t data);
    f_if.driver_cb.data_in <= data;
    f_if.driver_cb.push <= 1'b1;
    f_if.driver_cb.pop  <= 1'b0;
    @ ( f_if.driver_cb);
    f_if.driver_cb.push <= 1'b0;
  endtask : push_task 

  task Fifo_cmd_xactor::pop_task;
    f_if.driver_cb.pop  <= 1'b1;
    f_if.driver_cb.push <= 1'b0;        
    @ ( f_if.driver_cb);
    f_if.driver_cb.pop  <= 1'b0;
  endtask : pop_task

  task Fifo_cmd_xactor::push_pop_task (word_t data);
    f_if.driver_cb.data_in <= data;
    f_if.driver_cb.push <= 1'b1;
    f_if.driver_cb.pop  <= 1'b1;
    @ ( f_if.driver_cb);
    f_if.driver_cb.push <= 1'b0;
    f_if.driver_cb.pop  <= 1'b0;       
  endtask : push_pop_task     

  task Fifo_cmd_xactor::idle_task (int num_idle_cycles);
    begin
      a_0 : assert (num_idle_cycles < 10) else 
        `vmm_warning (log, 
		       $psprintf("Idle_task is invoked with LARGE number of idle cycles %0d ", num_idle_cycles)); 
      f_if.driver_cb.data_in <= 'X; // unsized Xs 
      f_if.driver_cb.push <= 1'b0;
      repeat (num_idle_cycles) @ ( f_if.driver_cb);
    end
  endtask : idle_task




 












