// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: Fifo_response
// PURPOSE: Provided the response from the command transactor to the generator

class Fifo_response extends vmm_data;
  import fifo_pkg::*;
  fifo_scen_e kind;
  word_t  data;	// data to push
  fifo_status_e status;  // PASSED, FAILED
  static vmm_log log = new("Fifo_response", "class");
  function new();
    super.new(this.log);
  endfunction : new

  extern virtual function string psdisplay(string prefix = "");

  extern virtual function  vmm_data copy(vmm_data cpy = null);

  extern virtual function Fifo_response allocate();

  extern virtual function bit compare(vmm_data   to,
                               output string diff,
                               input int kind = -1									  
                               );
endclass:Fifo_response

function string Fifo_response::psdisplay(string prefix);
    $sformat(psdisplay,
             "%s #%0d.%0d.%0d Fifo Xaction %s ",
             prefix, this.scenario_id, this.stream_id, this.data_id,
	     this.kind.name());
  endfunction : psdisplay

  
function  vmm_data Fifo_response::copy(vmm_data cpy);
  Fifo_response local_xactn;

  if (cpy == null) begin
   local_xactn  = new;
  end
  else if (!$cast(local_xactn, cpy)) begin
    `vmm_fatal(log, "Attempting to copy a non fifo_xactn instance");
    copy = null;
    return;
  end

  super.copy_data(local_xactn);
  local_xactn.kind = this.kind;
  local_xactn.data = this.data;
  copy = local_xactn;
endfunction : copy 

function Fifo_response Fifo_response::allocate();
    Fifo_response fifo_xactn_local=new();
    allocate = fifo_xactn_local;
endfunction : allocate 

function bit Fifo_response::compare(vmm_data   to,
                               output string diff,
                               input int kind
                               );
begin
   Fifo_response cmp;
   string tmp_str;
 
   compare = 1; // Assume success by default.
   diff    = "";
 
   // Cast assign the vmm_data handle to an Fifo_response handle
   if (!$cast(cmp, to)) begin
      `vmm_fatal(this.log, "Attempting to compare to a non Fifo_response instance");
      compare = 0;
      diff = "Cannot compare non Fifo_response to Fifo_response";
      return;
   end
 
   // Compare the individual data members, and set compare to 0 and
   // the "diff" text string to "xxx field mismatched" on failure.
 
   if (this.data !== cmp.data) begin compare = 0; diff = {diff, "Data Mismatch "}; end
   if (compare == 0)begin
     diff = {"Fifo_response objects are not identical. Mismatched field(s) is(are):\n", diff};
   end
end
endfunction

    
      // This macro declares new derived class named:
      // fifo_xactn_channel from vmm_channel
`vmm_channel (Fifo_response)

