// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: fifo_if
// PURPOSE: Defines the fifo interface used by the RTL and the
//          testbench 
interface fifo_if(input wire clk, 
                  input wire reset_n);
   timeunit 1ns;
   timeprecision 100ps;
  import fifo_pkg::*;
  logic  push; // push data into the fifo
  logic  pop;  // pop data from the fifo
  wire  full;  // fifo is at maximum level
  wire  empty; // fifo is at the zero level (no data)
  logic almost_empty, almost_full;
  
  // wire  error; // fifo push or pop error
  logic  error; // fifo push or pop error   
  word_t data_in;
  wire_word_t data_out;
  parameter hold_time=3;  // 3ns  
  parameter setup_time = 5;


  clocking slave_cb @ (posedge clk);
   default input #5ns output #hold_time;
     output empty, full, data_out, error;
     input  data_in, push, pop;
  endclocking : slave_cb


  clocking driver_cb @ (posedge clk);
   default input #setup_time output #hold_time;
     input  empty, full, data_out, error;
     output data_in, push, pop;
  endclocking : driver_cb
    
  // FIFO DUV 
  modport fslave_if_mp (clocking slave_cb,
                        output empty,
                        output full,
                        output data_out,
                        output error,
			output almost_empty,
			output almost_full,
                        input  data_in,
                        input  push,
                        input  pop);
  // input  clk);

  // FIFO driver 
   modport fdrvr_if_mp (clocking driver_cb);

// FIFO Monitor if.

 clocking mon_cb @ (posedge clk);
   default input #setup_time output #hold_time;
     input  empty, full, data_out, error;
     input data_in, push, pop;
  endclocking : mon_cb
 
  modport fifo_mon_if_mp (clocking mon_cb);
   
endinterface : fifo_if
