// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: Fifo_xactn
// PURPOSE: transaction generated by generator 

class Fifo_xactn extends vmm_data;
  import fifo_pkg::*;
  rand fifo_scen_t kind;
  rand word_t  data;	// data to push
  rand bit [7:0] idle_cycles;
  rand bit [7:0] reset_cycles;
  time xactn_time;
  static vmm_log log = new("Fifo_xactn", "class");
  
  constraint cst_data {
      data  < 1024;
  }
 constraint cst_idle {
   idle_cycles inside {[1:3]}; 
  }  
 constraint cst_reset {
   reset_cycles inside {[1:10]}; 
  }  

  constraint cst_xact_kind {
    kind dist {
      PUSH := 400,
      POP := 300,
      PUSH_POP :=200,
      IDLE := 30, 
      RESET := 10
    };
  } // cst_xact_kind

  function new();
    super.new(this.log);
  endfunction : new

  extern virtual function string psdisplay(string prefix = "");

  extern virtual function  vmm_data copy(vmm_data cpy = null);

  extern virtual function Fifo_xactn allocate();

  extern virtual function bit compare(vmm_data   to,
                               output string diff,
                               input int kind = -1									  
                               );
endclass:Fifo_xactn

function string Fifo_xactn::psdisplay(string prefix);
    $sformat(psdisplay,
             "%s #%0d.%0d.%0d Fifo Xaction %s ",
             prefix, this.scenario_id, this.stream_id, this.data_id,
	     this.kind.name());

    if (this.kind == IDLE) 
      $sformat(psdisplay, "%s Cycles %0d ", psdisplay, this.idle_cycles);
    if (this.kind == RESET) 
      $sformat(psdisplay, "%s Cycles %0d ", psdisplay, this.reset_cycles);
  
  endfunction : psdisplay

  
function  vmm_data Fifo_xactn::copy(vmm_data cpy);
  Fifo_xactn local_xactn;

  if (cpy == null) begin
   local_xactn  = new;
  end
  else if (!$cast(local_xactn, cpy)) begin
    `vmm_fatal(log, "Attempting to copy a non fifo_xactn instance");
    copy = null;
    return;
  end

  super.copy_data(local_xactn);
  local_xactn.kind = this.kind;
  local_xactn.data = this.data;
  local_xactn.idle_cycles = this.idle_cycles;
  copy = local_xactn;
endfunction : copy 

function Fifo_xactn Fifo_xactn::allocate();
    Fifo_xactn fifo_xactn_local=new();
    allocate = fifo_xactn_local;
endfunction : allocate 

function bit Fifo_xactn::compare(vmm_data   to,
                               output string diff,
                               input int kind
                               );
begin
   Fifo_xactn cmp;
   string tmp_str;
 
   compare = 1; // Assume success by default.
   diff    = "";
 
   // Cast assign the vmm_data handle to an Fifo_xactn handle
   if (!$cast(cmp, to)) begin
      `vmm_fatal(this.log, "Attempting to compare to a non Fifo_xactn instance");
      compare = 0;
      diff = "Cannot compare non Fifo_xactn to Fifo_xactn";
      return;
   end
 
   // Compare the individual data members, and set compare to 0 and
   // the "diff" text string to "xxx field mismatched" on failure.
 
   if (this.data !== cmp.data) begin compare = 0; diff = {diff, "Data Mismatch "}; end
   if (compare == 0)begin
     diff = {"Fifo_xactn objects are not identical. Mismatched field(s) is(are):\n", diff};
   end
end
endfunction

    
      // This macro declares new derived class named:
      // fifo_xactn_channel from vmm_channel
`vmm_channel (Fifo_xactn)
// ----------
class Fifo_xactn_no_push extends Fifo_xactn;
  constraint cst_xact_kind {
    kind dist {
      PUSH := 0,
      POP := 25,
      PUSH_POP :=0,
      IDLE := 3, 
      RESET := 1
    };
  } // cst_xact_kind

 extern virtual function  Fifo_xactn allocate();			  
endclass : Fifo_xactn_no_push

function Fifo_xactn Fifo_xactn_no_push::allocate();
    Fifo_xactn_no_push fifo_xactn_local=new();
	// `vmm_note(log, "allocated fifo_xactn_local");
    allocate = fifo_xactn_local;
endfunction : allocate 

// ----------
class Fifo_xactn_no_pop extends Fifo_xactn;
  constraint cst_xact_kind {
    kind dist {
      PUSH := 25,
      POP := 0,
      PUSH_POP :=0,
      IDLE := 3, 
      RESET := 1
    };
  } // cst_xact_kind

 extern virtual function  Fifo_xactn allocate();			  
endclass : Fifo_xactn_no_pop

function Fifo_xactn Fifo_xactn_no_pop::allocate();
    Fifo_xactn_no_pop fifo_xactn_local=new();
	// `vmm_note(log, "allocated fifo_xactn_local");
    allocate = fifo_xactn_local;
endfunction : allocate    