// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: fifo_env
// PURPOSE: Creates the verification environment 
class Fifo_env extends vmm_env;
  Fifo_cmd_xactor fifo_cmd_xactor_0;  // command-layer declaration
  Fifo_xactn_channel fifo_channel_0, fifo_mon_chan_0; // channel declaration
  Fifo_response_channel fifo_response_chan0;  // response channel cmd transactor -> generator
  Test_cfg test_cfg_0;  // Configuration declaration
  Fifo_xactn_atomic_gen fifo_xactn_gen_0;  // atomic generator declaration
  Fifo_mon_xactor mon_0;  // monitor declaration
  fifo_log_fmt log_fmt_cntl;  // format control declaration
  
  function new();   // for environment
	super.new();
    $timeformat(-9,2, " ns");
	this.test_cfg_0 = new;
    log_fmt_cntl = new();
    log = new("fifo_env", "");
    log.set_format(log_fmt_cntl);
  endfunction : new

  extern function void gen_cfg();	
  extern function void build();
  extern task reset_dut();
  extern task start();
  extern task wait_for_end();
  extern task report();
endclass : Fifo_env

  function void Fifo_env::gen_cfg();
    super.gen_cfg();
    this.test_cfg_0.randomize();
  endfunction : gen_cfg
  
  function void Fifo_env::build();
  string msg;
    super.build();
	`vmm_trace(log, "doing build");
    // Instantiation of channel  
    this.fifo_channel_0 = new("fifo_chan","channel");
    this.fifo_response_chan0=new("fifo_response_channel", "channel");
    // Instantiation of command-layer transactor 
    this.fifo_cmd_xactor_0 = new("cmd_xactor",
                        0,
                        `TOP.f_if, 
                        fifo_channel_0,
                        fifo_response_chan0
                        );
    // Instantiation of transaction generator 
    this.fifo_xactn_gen_0 = new ("fifo_gen", 0, fifo_channel_0);
    // Setting up the STOP 
    this.fifo_xactn_gen_0.stop_after_n_insts = this.test_cfg_0.no_of_xactions;
    // Setting up a message, then send it
    $sformat(msg, "Sim shall run for no_of_xactions %0d ", 
	   this.fifo_xactn_gen_0.stop_after_n_insts);
    `vmm_note(log, msg);
    // Instantiation of monitor channel
    this.fifo_mon_chan_0 = new("Fifo_mon_chan_0","channel");
    // Instantiation of monitor
    this.mon_0 = new("FIFO Mon", 0, `TOP.f_if,
                   this.fifo_mon_chan_0);
    // Setting up the fifo_xactn for the monitor
    // this.mon_0.factory_xactn=fifo_xactn;
	`vmm_trace(log, "end of build");
endfunction : build

  task Fifo_env:: reset_dut();
    super.reset_dut();
    `TOP.reset_n <= 1'b0;
    `TOP.f_if.pop   <= 1'b0;
    `TOP.f_if.push  <= 1'b0;	
    repeat (10) @(posedge `TOP.f_if.clk);
    `TOP.reset_n  <= 1'b1;
    repeat (10) @(posedge `TOP.f_if.clk);
  endtask : reset_dut

  task Fifo_env:: start();
    super.start();
    this.fifo_xactn_gen_0.start_xactor();
    this.fifo_cmd_xactor_0.start_xactor();
    this.mon_0.start_xactor();
  endtask : start
  
  task Fifo_env:: wait_for_end();
    super.wait_for_end();
    this.fifo_mon_chan_0.sink(); // flush content of channel 	
    this.fifo_xactn_gen_0.notify.wait_for(Fifo_xactn_atomic_gen::DONE);
  endtask : wait_for_end

      
 task Fifo_env:: report();
    super.report();
    `vmm_trace(log, "This is where additional model info is displayed");
	`vmm_note(log, "**** REPORT ***");
 endtask : report


