// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: fifo_cmd_xactor
// PURPOSE: Transactor for the fifo

class Fifo_cmd_xactor extends vmm_xactor;
  import fifo_pkg::*;
  word_t ldata;
  word_t cbdata=0; // for callback 
  Data_err_inject factory;
  Data_err_inject err_inject;


  virtual fifo_if.fdrvr_if_mp f_if;
  Fifo_xactn_channel in_chan;
  function new(        string               instance_s,
                       int unsigned         stream_id = -1,
               virtual fifo_if.fdrvr_if_mp  new_vir_if,
               Fifo_xactn_channel      new_in_chan);
      super.new("Fifo COMMAND Layer Xactor", instance_s, stream_id);
      this.f_if = new_vir_if;
      this.in_chan = new_in_chan;
  endfunction : new
  
  task main();
    //fork
      super.main();
    //join_none
    err_inject = factory.allocate(); 
   forever
    begin : main_loop
      Fifo_xactn  fifo_xactn;
      `vmm_note(log, "About to Get a new fifo xaction from in_channel ");
      this.in_chan.get(fifo_xactn);
    `vmm_note(log, $psprintf("Got a new fifo xaction from in_channel %s ",
                                  fifo_xactn.psdisplay()));
	 
    ldata = (fifo_xactn.data);
    case (fifo_xactn.kind)
      PUSH :
        begin : case_push
          if (err_inject.randomize()) begin : good_randomization 
	        if (err_inject.inject_err==INJECT) begin : inject_the_error
              `vmm_trace(log, "MUST DO THE FLIP_DATA THEN push_then_pop_task");
              `vmm_callback(Data_callback,flip_data(ldata));
    	       `vmm_trace(log, $psprintf("xaction.data= %h, flipped ldata= %h ",
                                  fifo_xactn.data, ldata));
 			  `vmm_note(log, "About to do a push_then_pop_task thru callback");
              `vmm_callback(Data_callback,  
                            push_then_pop_task(this, ldata, 3)); 
               // cbdata = cbdata +1;
            end : inject_the_error		  
            else this.push_task(ldata); // no error injection
          end : good_randomization
          else `vmm_trace(log, "error in err_inject.randomize");
        end : case_push
      POP  :
        begin
         this.pop_task(); 
        end
      PUSH_POP :
        begin
         this.push_pop_task(ldata);
        end
      IDLE :
        begin
          this.idle_task(fifo_xactn.idle_cycles);
        end
      RESET :
        this.reset_task(5); 
      endcase
    end : main_loop   
  endtask : main

  task reset_task (int num);
   @ ( f_if.driver_cb);
    for (int i=0; i < num; i++) 
    begin 
      `TOP.reset_n <= 1'b0;  
      idle_task(num); 
    end
    `TOP.reset_n <= 1'b1;
    @ ( f_if.driver_cb);
    @ ( f_if.driver_cb);	   
  endtask : reset_task

  virtual task push_task (word_t data
              );
     begin
       `vmm_trace(log, $psprintf ("Push data %0h ", data));
	   f_if.driver_cb.data_in <= data;
	   f_if.driver_cb.push <= 1'b1;
	   f_if.driver_cb.pop  <= 1'b0;
	   @ ( f_if.driver_cb);
       f_if.driver_cb.push <= 1'b0;
      end
  endtask : push_task 
     

  task pop_task;
    begin
      f_if.driver_cb.pop  <= 1'b1;
      f_if.driver_cb.push <= 1'b0;        
      @ ( f_if.driver_cb);
      f_if.driver_cb.pop  <= 1'b0;
    end
  endtask : pop_task

  task push_pop_task (word_t data);
     begin
       `vmm_trace(log, $psprintf ("%0t %m Push data %0h ", $time, data));
	   f_if.driver_cb.data_in <= data;
	   f_if.driver_cb.push <= 1'b1;
	   f_if.driver_cb.pop  <= 1'b1;
	   @ ( f_if.driver_cb);
       f_if.driver_cb.push <= 1'b0;
       f_if.driver_cb.pop  <= 1'b0;       
      end
  endtask : push_pop_task     

  task idle_task (int num_idle_cycles);
    begin
      a_0 : assert (num_idle_cycles < 4) else 
        $warning ( "%0t %0m idle_task is invoked with LARGE number of idle cycles %0d ",
                  $time, num_idle_cycles); 
      `vmm_trace(log, $psprintf("idle_task #cycles= %0d ",
                   num_idle_cycles)); 
      f_if.driver_cb.data_in <= 'X; // unsized Xs 
      f_if.driver_cb.push <= 1'b0;
      repeat (num_idle_cycles) @ ( f_if.driver_cb);
    end
  endtask : idle_task

endclass: Fifo_cmd_xactor



 












