// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: fifo_env
// PURPOSE: Creates the verification environment
class Fifo_env extends vmm_env;
  import fifo_pkg::*;
  Fifo_cmd_xactor fifo_cmd_xactor_0;
  Fifo_xactn_channel fifo_xactn_channel, fifo_mon_chan_0;
  Data_err_inject err_inject_0;  // new 
  virtual fifo_if vir_if;
  Data2_callback d2_cbk =new;
  Data3_callback d3_cbk =new;

  Push_cfg push_cfg_0;
  Fifo_xactn_atomic_gen push_gen_0;
  Fifo_mon_xactor mon_0;
  Fifo_log_fmt log_fmt_cntl;
  vmm_log log;
  
  function new();
	super.new();	
    $timeformat(-9,2, " ns");
    this.log_fmt_cntl = new();
    this.log = new("FIFO Env Logger", "0");
    this.log.set_format(this.log_fmt_cntl);
  endfunction : new

  extern function void build();
  extern task reset_dut();
  extern task start();
  extern task wait_for_end();
  extern task report();
endclass : Fifo_env
	
function void Fifo_env::build();
  string msg;
    super.build();
	this.err_inject_0 = new; // new	
    this.fifo_xactn_channel = new("push_chan","0");
    this.fifo_cmd_xactor_0 = new("PUSH_XACTOR",
                        0,
                         `TOP.f_if, 
                        fifo_xactn_channel
                        );
	// this.fifo_cmd_xactor_0.d_cb_factory=this.d_cb;
   this.fifo_cmd_xactor_0.factory = err_inject_0; // new 
	//registering the callback 
   this.fifo_cmd_xactor_0.append_callback(d2_cbk);
   this.fifo_cmd_xactor_0.append_callback(d3_cbk);   
	
    this.push_gen_0 = new ("Push Xaction generator", 0);
    this.push_cfg_0 = new;
    this.push_cfg_0.randomize();
    this.push_gen_0.stop_after_n_insts = this.push_cfg_0.no_of_xactions;
   $sformat(msg, "Sim shall run for no_of_xactions %0d ", this.push_gen_0.stop_after_n_insts);
    `vmm_note(this.log, msg);
    this.push_gen_0.out_chan = this.fifo_xactn_channel;

    // Monitor section
	this.fifo_mon_chan_0 = new("Fifo_mon_chan_0","channel");
    this.mon_0 = new("FIFO Mon", 0,  `TOP.f_if, 
                     this.fifo_mon_chan_0);

   //this.mon_push_chan.reconfigure(this.push_cfg_0.no_of_xactions + 1);
   
endfunction : build

task Fifo_env::reset_dut();
    super.reset_dut();
    `TOP.reset_n <= 1'b0;
    `TOP.f_if.pop   <= 1'b0;
    `TOP.f_if.push  <= 1'b0;	
	repeat (10) @(`TOP.f_if.driver_cb);
    `TOP.reset_n  <= 1'b1;
    repeat (10) @(`TOP.f_if.driver_cb);
endtask : reset_dut

task Fifo_env::start();
    super.start();
    this.push_gen_0.start_xactor();
    this.fifo_cmd_xactor_0.start_xactor();
    this.mon_0.start_xactor();
    this.fifo_mon_chan_0.sink(); // flush content of channel 	
 endtask : start
  
task Fifo_env::wait_for_end();
    super.wait_for_end();
    fork : f_gen
      this.push_gen_0.notify.wait_for(Fifo_xactn_atomic_gen::DONE);
    join : f_gen
    
endtask : wait_for_end

 task Fifo_env:: report();
    super.report();
    `vmm_trace(log, "This is where additional model info is displayed");
	`vmm_note(log, "**** REPORT ***");
 endtask : report
