// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: fifo_props
// Purpose:  PROPERTY MODULE for FIFO
// This module is used for verification of the FIFO, and is 
// intended to be bound in the top testbench (with the SystemVerilog "bind") to the DUV
module fifo_props (input clk, input reset_n, fifo_if fifo_if);
  timeunit 1ns;
  timeprecision 100ps;
  int size;
  import fifo_pkg::*;
  always @(posedge clk) begin
	if (!reset_n) size <= 0;
	if (size != FULL && fifo_if.push && !fifo_if.pop)  size <= size+1;
    if (size != 0 && fifo_if.pop && !fifo_if.push)  size <= size-1;
  end
  
// Activating reset during interesting corner cases
// is covered via cover directives.

  property p_t1_full;  @ (posedge clk) 
    fifo_if.full;  // ##1 reset_n==0;
  endproperty : p_t1_full
  cp_t1_full_1: cover property (p_t1_full);
  

  property p_t3_empty;  @ (posedge clk) 
    fifo_if.empty; // ##1 reset_n==0;
  endproperty : p_t3_empty
  cp_t3_empty_1: cover property (p_t3_empty);
  

  property p_push_pop_sequencing;
      @ (posedge clk) fifo_if.push |=> ##[0:$] fifo_if.pop;
  endproperty : p_push_pop_sequencing
     
  // coverage of sequences 
  cp_push_pop_sequencing  : cover property  (p_push_pop_sequencing);
 
  // never a push and full and no pop
  property p_push_error;
    @ (posedge clk)
       not (f_if.push && f_if.full && !f_if.pop);
  endproperty : p_push_error
  ap_push_error : assert property (p_push_error);

   // never a pop on empty
  property p_pop_error;
   @ (posedge clk)
       not (f_if.pop && f_if.empty);// bc -- increment only if OK 
  endproperty : p_pop_error
  ap_pop_error : assert property (p_pop_error);

  ap_full2empty : assert property (@(posedge clk) disable iff(!reset_n)
                                    not (f_if.full ##1 f_if.empty));


  // Flags at reset
  property p_fifo_ptrs_flags_at_reset;
    @ (posedge clk)
       !reset_n |-> !f_if.full && f_if.empty;
  endproperty  : p_fifo_ptrs_flags_at_reset
  ap_fifo_ptrs_flags_at_reset : assert property (p_fifo_ptrs_flags_at_reset);

  ap_fifo_full : assert property   (@ (posedge clk)  fifo_if.full    |-> (size == FULL));
  ap_fifo_full2 : assert property  (@ (posedge clk) (size == FULL) |-> fifo_if.full);
  ap_fifo_empty : assert property  (@ (posedge clk)  fifo_if.empty   |-> (size == 0));
  ap_fifo_empty2 : assert property (@ (posedge clk)  fifo_if.empty   |-> (size == 0));

endmodule : fifo_props   

 