// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: Fifo_custom_gen
// PURPOSE: custom generator 

class Fifo_custom_gen extends vmm_xactor;
  import fifo_pkg::*;
  Fifo_xactn_channel out_chan;
  Fifo_xactn         randomized_obj;
  Fifo_response_channel resp_chan;  // responsse  from cmd xactor
  int unsigned                stream_id = -1;
  function new(string                      inst,
               int unsigned                stream_id = -1,
               Fifo_xactn_channel          out_chan=null, 
               Fifo_response_channel       fifo_response_channel=null);
      super.new("Custom Fifo COMMAND Layer Xactor", inst, stream_id);
	  if (out_chan==null) out_chan=new("fifo_chanel", "channel");
	  else this.out_chan=out_chan;
   	  if (fifo_response_channel==null)
        this.resp_chan=new("fifo_response_chan", "channel");
      else this.resp_chan=fifo_response_channel;
      this.notify.configure(fifo_pkg::DONE_GEN, vmm_notify::ON_OFF);
      this.stream_id=stream_id;
  endfunction : new
  
  task main();
    `vmm_trace(log, "Inside Generator");
    fork
      super.main();
   join_none
	 if (randomized_obj==null) randomized_obj=new();
    this.gen(); 
    `vmm_note (log, "Gen is done");
    this.notify.indicate(fifo_pkg::DONE_GEN);	
  endtask : main

    extern task gen(); // generation of transactions 
    extern task push(word_t data);
    extern task pop();	
    extern task push_pop(word_t data);    
    extern task idle(int idle);
    extern task reset();
    extern task put_in_channel(Fifo_xactn obj);
endclass : Fifo_custom_gen

task Fifo_custom_gen::gen();
    word_t v_data=0;  // not used, shown for potential use. 
    string msg;
  	 `vmm_trace(log, "Generator creating directed tests: Reset, 4 push, 4 pop");
    this.reset();
    // Do 4 consecutive push with directed data 
    for (int i=0; i<4; i++) this.push(i);
    // do 4 pop 
    repeat(4) this.pop();
    // do rand streams
    `vmm_trace(log, "Generator creating stream randsequence tests");    
	for (int i=0; i<100; i++)
	  randsequence (stream)
        stream : first second third; // := 10 | second := 20 | third :=1;
          first : s_push | s_pop; 
          second : s_idle :=2 | s_push_pop := 7 | s_reset:= 1;
          third : s_idle:=8 | s_pop :=2;
          s_push         : {this.push(i*4);};           
          s_pop          : {this.pop();}; 
          s_idle         : {this.idle(1);}; 
          s_push_pop     : {this.push_pop(i*2);}; 
          s_reset        : {this.reset();}; 
        endsequence
      idle(1);
      //do random tests
	 `vmm_trace(log, "Generator creating random tests per transaction class constraints");
     repeat(100) begin : for_random_tests
       randomized_obj.randomize();
       this.put_in_channel(randomized_obj); 
     end : for_random_tests
    `vmm_note(log, "done tests"); 
endtask : gen 

// Special code for waiting for notification from command transactor
// Wait is for the ENDED and not the STARTED
task Fifo_custom_gen::push(word_t data);
     Fifo_response  bfm_response;
      `vmm_note(log, "PUSH");
      randomized_obj.data=data;
      randomized_obj.kind=PUSH;
      //this.out_chan.put(randomized_obj.copy(cur_xactn));
      this.put_in_channel(randomized_obj);
	  randomized_obj.notify.wait_for(vmm_data::ENDED);
      // get the BFM response
	  $cast(bfm_response, randomized_obj.notify.status(vmm_data::ENDED));
	  if (bfm_response.fifo_status==fifo_pkg::PASSED) 
	  `vmm_trace(log, "Cmd Xactor Ended the PUSH");
	  else `vmm_note(log, "Cmd Xactor FAILED in completing the PUSH");
    // must reset the notification
    // A vmm_notify::SOFT reset clears the specified ON_OFF notification and
    // restarts the vmm_notification::indicate() and vmm_notification::reset()
    // methods on any attached notification descriptor.
	// ***  NOT NEEDED - See book page 114 ***
	//  randomized_obj.notify.reset(vmm_data::ENDED, vmm_notify::SOFT);
	
endtask : push


task Fifo_custom_gen::pop();
      `vmm_note(log, "POP");
      randomized_obj.data=32'hX;
      randomized_obj.kind=POP;
      this.put_in_channel(randomized_obj);
endtask : pop  
  
task Fifo_custom_gen::push_pop(word_t data);
      `vmm_note(log, "PUSH_POP");
      randomized_obj.data=data;
      randomized_obj.kind=PUSH_POP;
      this.put_in_channel(randomized_obj);
endtask : push_pop

task Fifo_custom_gen::idle(int idle);
    $display("idle");
    randomized_obj.randomize(); 
    randomized_obj.kind=IDLE;
    randomized_obj.idle_cycles=idle;
    this.put_in_channel(randomized_obj);
endtask : idle

task Fifo_custom_gen::reset();
      `vmm_note(log, "RESET");
      randomized_obj.data=32'hX;
      randomized_obj.kind=RESET;
      this.put_in_channel(randomized_obj);
endtask : reset
  
task Fifo_custom_gen::put_in_channel(Fifo_xactn obj);
      Fifo_xactn fifo_inst;
      obj.data_id++;
      $cast(fifo_inst, obj.copy(obj));
      this.out_chan.put(fifo_inst);
endtask : put_in_channel
