// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: fifo_env
// PURPOSE: Creates the verification environment 

class Fifo_env extends vmm_env;
  import fifo_pkg::*;
  Fifo_xactn fifo_xactn; // fifo transaction class declaration 
  Fifo_cmd_xactor fifo_cmd_xactor_0;  // command-layer declaration
  Fifo_xactn_channel fifo_channel_0, fifo_mon_chan_0; // channel declaration
  Fifo_response_channel fifo_response_chan0;  // response channel cmd transactor -> generator
//  Test_cfg test_cfg_0;  // Configuration declaration
//  Fifo_xactn_atomic_gen xactn_gen_0;  // atomic generator declaration
  Fifo_mon_xactor mon_0;  // monitor declaration
  Fifo_log_fmt log_fmt_cntl;  // format control declaration
  vmm_log log;  // log declaration

//  push_cfg push_cfg_0;
//  fifo_xactn_atomic_gen xactn_gen_0;
  Fifo_custom_gen fifo_xactn_gen_0;
//  Fifo_xactn  fifo_xactn;  // one mix of constraints 
  Fifo_xactn2 next_xactn;  // unused
  
  function new();   // for environment
    super.new();	
    $timeformat(-9,2, " ns");
    this.log_fmt_cntl = new();
    this.log = new("FIFO Env Logger", "0");
    this.log.set_format(this.log_fmt_cntl);
  endfunction : new // unmatched end(function|task|module|primitive)

  extern function void build();
  extern task reset_dut();
  extern task start();
  extern task wait_for_end();
  extern task report();
endclass : Fifo_env	
// --------------------
function void Fifo_env::build();
  string msg;
    super.build();
    // Instantiation of channel  
    this.fifo_channel_0 = new("fifo_chan","channel");
    this.fifo_response_chan0=new("fifo_response_channel", "channel");
    // Instantiation of command-layer transactor 
    this.fifo_cmd_xactor_0 = new("cmd_xactor",
                        0,
                        `TOP.f_if, 
                        fifo_channel_0,
                        fifo_response_chan0
                        );
    // Instantiation of transaction generator 
    this.fifo_xactn_gen_0 = new ("fifo_gen", 0, fifo_channel_0, fifo_response_chan0);
    // Instantiation of transaction class 
    fifo_xactn = new();     
    // Setting the factory for the transaction on the generator (see chapter 5 for factory)
    // ** NOTE: If next line is commented out, then transaction generator 
    //          will use its internally instantiated copy of randomized_obj
    fifo_xactn_gen_0.randomized_obj = fifo_xactn;

    // Instantiation of monitor channel
    this.fifo_mon_chan_0 = new("Fifo_mon_chan_0","channel");
    // Instantiation of monitor
    this.mon_0 = new("FIFO Mon", 0, `TOP.f_if,
                   this.fifo_mon_chan_0);
    // Setting up the fifo_xactn for the monitor
    this.mon_0.factory_xactn=fifo_xactn;

endfunction : build



  task Fifo_env:: reset_dut();
    super.reset_dut();
    `TOP.reset_n <= 1'b0;
    `TOP.f_if.pop   <= 1'b0;
    `TOP.f_if.push  <= 1'b0;	
    repeat (10) @(posedge `TOP.f_if.clk);
    `TOP.reset_n  <= 1'b1;
    repeat (10) @(posedge `TOP.f_if.clk);
  endtask : reset_dut
  

  task Fifo_env::start();
    super.start();
    this.fifo_xactn_gen_0.start_xactor();
    this.fifo_cmd_xactor_0.start_xactor();
    this.mon_0.start_xactor();
	this.fifo_mon_chan_0.sink(); // flush content of channel 
	//wait for end of 1st sets of tests
  endtask : start

  task Fifo_env:: wait_for_end();
    super.wait_for_end();
    this.fifo_xactn_gen_0.notify.wait_for(fifo_pkg::DONE_GEN);
	`vmm_note(this.log, "*** Completed transactions");	
  endtask : wait_for_end

 task Fifo_env:: report();
    super.report();
    `vmm_trace(log, "This is where additional model info is displayed");
	`vmm_note(log, "**** REPORT ***");
 endtask : report
