// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: eth_boradcaster_env
// PURPOSE: Creates the verification environment


`include "eth_log_fmt.sv"
`include "eth_xactn.sv"
`include "eth_atomic_gen.sv"  
`include "eth_asink.sv"  
class Eth_env extends vmm_env;
  
  Eth_xactn_channel eth_src_channel;
  Eth_xactn_channel eth_des_channel_array[];
  Eth_asink eth_asink_array[];
  
  vmm_broadcast eth_broadcaster;

  Test_cfg test_cfg_0;
  Eth_xactn_atomic_gen eth_atom_gen_0;
  Eth_log_fmt log_fmt_cntl;
  vmm_log log;
  
  function new();
    $timeformat(-9,2, " ns");
    this.log_fmt_cntl = new();
    this.log = new("ETH Env Logger", "0");
    this.log.set_format(this.log_fmt_cntl);
  endfunction : new

  extern function void gen_cfg();	
  extern function void build();
  extern task start();
  extern task wait_for_end();
endclass : Eth_env

 function void Eth_env::gen_cfg();  
  super.gen_cfg();
    this.test_cfg_0 = new;
    this.test_cfg_0.randomize();
    `vmm_note(this.log, $psprintf("%s %0d %s",
				  "Sim shall run for ",
				  this.test_cfg_0.no_of_xactions,
				  "transactions"
				  ));
endfunction : gen_cfg
  
  function void Eth_env::build();
  string msg;
    super.build();
    this.eth_src_channel = new("eth_chan","0");

    this.eth_atom_gen_0 = new("Atomic Gen", "",eth_src_channel);
    this.eth_atom_gen_0.stop_after_n_insts = this.test_cfg_0.no_of_xactions;

    this.eth_broadcaster = new(
			     "ETH broadcaster", // name
			     "0",                  // instance
			       eth_src_channel);      // source
			      
  
 endfunction : build

task Eth_env::start();
  super.start();
  this.eth_atom_gen_0.start_xactor();
  this.eth_broadcaster.start_xactor();
  foreach (eth_asink_array [iter]) begin : fe_l_1
    this.eth_asink_array[iter].start_xactor();
  end : fe_l_1
endtask : start
  
task Eth_env::wait_for_end();
  super.wait_for_end();
  this.eth_atom_gen_0.notify.wait_for(Eth_xactn_atomic_gen::DONE);
  #10000;
  `vmm_note(this.log, "Reached EOT!");
endtask : wait_for_end
