// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: eth_xactn
// PURPOSE: Transaction sent by generator

class Eth_xactn extends vmm_data;

  rand logic [31:0] pkt_pld;
  rand logic err_pkt;
  rand bit [7:0] pkt_length;
  
  time xactn_time;
  
  static vmm_log log = new("eth_xactn", "");

  function new();
    super.new(this.log);
  endfunction : new

  extern virtual function string psdisplay(string prefix = "");
  extern virtual function  vmm_data copy(vmm_data cpy = null);
  extern virtual function vmm_data allocate();
  extern virtual function bit compare(vmm_data   to,
                               output string diff,
                               input int kind = -1		  
                               );
endclass : Eth_xactn

function string Eth_xactn::psdisplay(string prefix );
  $sformat(psdisplay,
           "%s #%0d.%0d.%0d Ethernet %s Pkt, length %0d, data %0h",
           prefix, this.stream_id, this.scenario_id, this.data_id,
	   this.err_pkt ? "ERROR" : "GOOD",
	   this.pkt_length, this.pkt_pld);
endfunction : psdisplay

function  vmm_data Eth_xactn::copy(vmm_data cpy);
Eth_xactn local_xactn;

  if (cpy == null) begin
   local_xactn  = new;
  end
  else if (!$cast(local_xactn, cpy)) begin
    `vmm_fatal(log, "Attempting to copy a non eth_xactn instance");
    copy = null;
    return;
  end
  super.copy_data(local_xactn);

  local_xactn.pkt_pld = this.pkt_pld;
  local_xactn.err_pkt = this.err_pkt;
  local_xactn.pkt_length = this.pkt_length;
  return(local_xactn);
endfunction : copy


function vmm_data Eth_xactn::allocate();
  Eth_xactn eth_xactn_local=new();
  allocate = eth_xactn_local;
endfunction : allocate 

function bit Eth_xactn::compare(vmm_data   to,
                               output string diff,
                               input int kind
                               );
   Eth_xactn cmp;
   string tmp_str;
 
   compare = 1; // Assume success by default.
   diff    = "";
 
   // Cast assign the vmm_data handle to an Eth_xactn handle
   if (!$cast(cmp, to)) begin
      `vmm_fatal(this.log, "Attempting to compare to a non Eth_xactn instance");
      compare = 0;
      diff = "Cannot compare non Eth_xactn to Eth_xactn";
      return;
   end
 
   // Compare the individual data members, and set compare to 0 and
   // the "diff" text string to "xxx field mismatched" on failure.
   if (this.pkt_pld !== cmp.pkt_pld) begin : if_0
     compare = 0; 
     diff = $psprintf("%s Payload Mismatch Expected %0x Actual %0x", diff,
		      this.pkt_pld, cmp.pkt_pld);
   end : if_0 
   if (this.pkt_length !== cmp.pkt_length) begin : if_1
     compare = 0; 
     diff = $psprintf("%s Length Mismatch Expected %0x Actual %0x", diff,
		      this.pkt_length, cmp.pkt_length);
   end : if_1 
 
  if (compare == 0)
  begin
    diff = {"Eth_xactn objects are not identical. Mismatched field(s) is(are):\n", diff};
  end
endfunction : compare

`vmm_channel (Eth_xactn)

