// VhdlCohen Publishing, copyright 2006 
// wwww.abv-sva.org   ben@abv-sva.org  01/05/06  
// NAME: fifo_cmd_xactor
// PURPOSE: Transactor for the fifo
// Note: The fifo interface is virtual
//       The clock will be extracted from the fifo interface
//       The reset is driven by this transactor, and the reset signal
//        transferred as an argument to the program is driven
//        by this transactor, and is not using a virtual interface.
class Fifo_cmd_xactor extends vmm_xactor;
  import fifo_pkg::*;
  virtual fifo_if.fdrvr_if_mp f_if;
  vmm_log log;
  Fifo_xactn_channel in_chan;
  function new(string instance,
	       int unsigned stream_id = -1,
               virtual fifo_if.fdrvr_if_mp  new_vir_if,
               Fifo_xactn_channel      new_in_chan = null);
      super.new("cmd_xactor", instance, stream_id);
      this.f_if = new_vir_if;
      this.in_chan = new_in_chan;
      this.log = new("cmd_xactor","");
  endfunction : new

  extern task main();
  extern task reset_task (int num);
  extern task push_task (word_t data);
  extern task pop_task();
  extern task push_pop_task (word_t data);
  extern task idle_task (int num_idle_cycles);
endclass: Fifo_cmd_xactor

typedef class Fifo_fcov_cb;
  
  task Fifo_cmd_xactor::main();
    fork
      super.main();
    join_none
    forever
    begin : main_loop
      Fifo_xactn  fifo_xactn_0;
      this.in_chan.get(fifo_xactn_0);
    `vmm_note(this.log, $psprintf("Got a new fifo xaction from in_channel %s ",
                                  fifo_xactn_0.psdisplay()));
    case (fifo_xactn_0.kind)
      PUSH :
        begin
          this.push_task(fifo_xactn_0.data);
        end
      POP  :
        begin
         this.pop_task(); 
        end
      PUSH_POP :
        begin
         this.push_pop_task(fifo_xactn_0.data);
        end
      IDLE :
        begin
          this.idle_task(fifo_xactn_0.idle_cycles);
        end
      RESET :
        this.reset_task(fifo_xactn_0.reset_cycles); 
      endcase
  `vmm_callback (Fifo_fcov_cb,
		   post_trans(this, fifo_xactn_0));
   end : main_loop   
  endtask : main

  task Fifo_cmd_xactor::reset_task (int num);
    @ ( f_if.driver_cb);
    for (int i=0; i < num; i++) 
    begin 
      reset_n <= 1'b0;  
      idle_task(num); 
    end
    reset_n <= 1'b1;
    @ ( f_if.driver_cb);
    @ ( f_if.driver_cb);	   
  endtask : reset_task
     
  task Fifo_cmd_xactor::push_task (word_t data);
    f_if.driver_cb.data_in <= data;
    f_if.driver_cb.push <= 1'b1;
    f_if.driver_cb.pop  <= 1'b0;
    @ ( f_if.driver_cb);
    f_if.driver_cb.push <= 1'b0;
  endtask : push_task 

  task Fifo_cmd_xactor::pop_task;
    f_if.driver_cb.pop  <= 1'b1;
    f_if.driver_cb.push <= 1'b0;        
    @ ( f_if.driver_cb);
    f_if.driver_cb.pop  <= 1'b0;
  endtask : pop_task

  task Fifo_cmd_xactor::push_pop_task (word_t data);
    f_if.driver_cb.data_in <= data;
    f_if.driver_cb.push <= 1'b1;
    f_if.driver_cb.pop  <= 1'b1;
    @ ( f_if.driver_cb);
    f_if.driver_cb.push <= 1'b0;
    f_if.driver_cb.pop  <= 1'b0;       
  endtask : push_pop_task     

  task Fifo_cmd_xactor::idle_task (int num_idle_cycles);
    begin
      a_0 : assert (num_idle_cycles < 10) else 
        `vmm_warning ( this.log, 
		       $psprintf("Idle_task is invoked with LARGE number of idle cycles %0d ", num_idle_cycles)); 
      f_if.driver_cb.data_in <= 'X; // unsized Xs 
      f_if.driver_cb.push <= 1'b0;
      repeat (num_idle_cycles) @ ( f_if.driver_cb);
    end
endtask : idle_task


  




 












