// VhdlCohen Publishing, copyright 2006 
// wwww.abv-sva.org   ben@abv-sva.org  01/05/06  
// NAME: fifo_cov_model
// PURPOSE: Coverage model for the fifo

class Fifo_cov_model;
  import fifo_pkg::*;

  logic  push; // push data into the fifo
  logic  pop;  // pop data from the fifo
  logic  full;  // fifo is at maximum level
  logic  empty; // fifo is at the zero level (no data)
  logic almost_empty, almost_full;
  logic  error; // fifo push or pop error   
  word_t data;
endclass : Fifo_cov_model

class Fifo_fcov_plug_in extends Fifo_fcov_cb;
  
  Fifo_cov_model fifo_cov_model_0;
  vmm_log log;
  virtual fifo_if.fifo_mon_if_mp vif;
  
  function new(virtual fifo_if.fifo_mon_if_mp new_vif);
    this.log = new ("fcov_logger", "0");
    this.fifo_cov_model_0 = new;
    this.full_cg = new();
    this.full_push_pop_cg = new();
    this.empty_cg = new();
   // this.full_push_cg = new();
    this.vif = new_vif;
  endfunction : new
  
  covergroup full_push_pop_cg;
  full_cpt : coverpoint fifo_cov_model_0.full;
  push_cpt : coverpoint fifo_cov_model_0.push;
  pop_cpt  : coverpoint fifo_cov_model_0.pop;

  pp_during_full_cr : cross full_cpt,push_cpt,pop_cpt;
  
  endgroup : full_push_pop_cg

  covergroup full_cg;
  fifo_full_cpt : coverpoint fifo_cov_model_0.full iff (reset_n);
  endgroup : full_cg

  covergroup empty_cg;
  fifo_empty_cpt : coverpoint fifo_cov_model_0.empty iff (reset_n);
  endgroup : empty_cg
    
 extern function void post_trans(Fifo_cmd_xactor bfm, 
				   Fifo_xactn  fifo_xactn_0);
endclass : Fifo_fcov_plug_in
 
function void Fifo_fcov_plug_in::post_trans(Fifo_cmd_xactor bfm, 
				   Fifo_xactn  fifo_xactn_0);

  `vmm_note (this.log, $psprintf ("In post_trans, sampling push %0b, full %0b",
				  vif.mon_cb.push,
				  vif.mon_cb.full));
  
  this.fifo_cov_model_0.full = this.vif.mon_cb.full;
  this.fifo_cov_model_0.push = this.vif.mon_cb.push;
  this.fifo_cov_model_0.pop = this.vif.mon_cb.pop;
  this.fifo_cov_model_0.empty = this.vif.mon_cb.empty;
  this.full_cg.sample();
  this.empty_cg.sample();
  this.full_push_pop_cg.sample();
  //this.full_push_cg.sample();
endfunction : post_trans