// VhdlCohen Publishing, copyright 2006 
// wwww.abv-sva.org   ben@abv-sva.org  01/05/06  
// NAME: fifo_env
// PURPOSE: Creates the verification environment 
`include "fifo_xactn.sv"
`include "fifo_log_fmt.sv"
`include "fifo_cmd_xactor.sv"
`include "fifo_gen_xactor.sv"  
`include "fifo_mon_xactor.sv"  
`include "fifo_fcov_cb_defs.sv"
`include "fifo_cov_model.sv"

class Fifo_env extends vmm_env;
  Fifo_cmd_xactor fifo_cmd_xactor_0;
  Fifo_xactn_channel fifo_channel_0, fifo_mon_chan_0;

  Test_cfg test_cfg_0;
  Fifo_xactn_atomic_gen fifo_xactn_gen_0;
  Fifo_mon_xactor mon_0;
  Fifo_log_fmt log_fmt_cntl;
  vmm_log log;
  Fifo_fcov_plug_in fifo_fcov_plug_in_0;
  
  function new();
    $timeformat(-9,2, " ns");
    this.log_fmt_cntl = new();
    this.log = new("fifo_env", "");
    this.log.set_format(this.log_fmt_cntl);
  endfunction : new

  extern function void build();
  extern task reset_dut();
  extern task start();
  extern task wait_for_end();
  
endclass : Fifo_env
  
function void Fifo_env::build();
string msg;
super.build();
this.fifo_channel_0 = new("fifo_chan","");
this.fifo_cmd_xactor_0 = new("cmd_xactor",
                             0,
                             `TOP.f_if, 
                             fifo_channel_0
                             );

this.fifo_xactn_gen_0 = new ("fifo_gen", 0);
this.test_cfg_0 = new;
this.test_cfg_0.randomize();
this.fifo_xactn_gen_0.stop_after_n_insts = this.test_cfg_0.no_of_xactions;
$sformat(msg, "Sim shall run for no_of_xactions %0d ", 
	 this.fifo_xactn_gen_0.stop_after_n_insts);
`vmm_note(this.log, msg);
this.fifo_xactn_gen_0.out_chan = this.fifo_channel_0;

// Monitor section
this.fifo_mon_chan_0 = new("Fifo_mon_chan_0","0");
this.mon_0 = new("FIFO Mon", 0, `TOP.f_if,
                 this.fifo_mon_chan_0);

// Coverage model
this.fifo_fcov_plug_in_0 = new(`TOP.f_if.fifo_mon_if_mp);
this.fifo_cmd_xactor_0.append_callback(this.fifo_fcov_plug_in_0);
endfunction : build

task Fifo_env:: reset_dut();
super.reset_dut();
reset_n <= 1'b0;
`TOP.f_if.pop   <= 1'b0;
`TOP.f_if.push  <= 1'b0;	
repeat (10) @(posedge `TOP.f_if.clk);
reset_n  <= 1'b1;
repeat (10) @(posedge `TOP.f_if.clk);
endtask : reset_dut

task Fifo_env:: start();
super.start();
this.fifo_xactn_gen_0.start_xactor();
this.fifo_cmd_xactor_0.start_xactor();
this.mon_0.start_xactor();
endtask : start

task Fifo_env:: wait_for_end();
super.wait_for_end();
this.fifo_xactn_gen_0.notify.wait_for(Fifo_xactn_atomic_gen::DONE);
#10000; // needs a better fix
endtask : wait_for_end

      

