// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: Fifo_mon_xactor
// PURPOSE: Provided the monitor function of transactions seen on the Fifo interface
class Fifo_mon_xactor extends vmm_xactor;
  virtual fifo_if f_if;
  vmm_log log;
  Fifo_xactn_channel push_out_chan;
  Fifo_xactn cur_push_xactn;

  function new(string               instance,
             int unsigned         stream_id = -1,
             virtual fifo_if new_vir_if,
             Fifo_xactn_channel     new_push_out_chan);
      super.new("Fifo COMMAND Layer Xactor", instance, stream_id);
      this.f_if = new_vir_if;
      this.log = new("FIFO_MON","0");
      this.push_out_chan = new_push_out_chan;
  endfunction : new
  
  extern task main();
  extern task mon_push();
endclass : Fifo_mon_xactor

task Fifo_mon_xactor::main();
    `vmm_note(this.log, "Inside Monitor");
    fork
      super.main();
      this.mon_push();
   join_none
  endtask : main

task Fifo_mon_xactor::mon_push();
string msg;
  while (1) begin : mon_push_loop
    @(this.f_if.mon_cb);
    if (this.f_if.mon_cb.push === 1'b1) begin
      this.cur_push_xactn = new();
      this.cur_push_xactn.data = this.f_if.mon_cb.data_in;
      this.cur_push_xactn.xactn_time = $time;
      $sformat(msg,"Found a PUSH Xactn at time %0t data %0d ",
               $time, this.cur_push_xactn.data);
      `vmm_note(this.log, msg);
 
      this.push_out_chan.put(this.cur_push_xactn);
    end // if
    else
    begin
      $sformat(msg," waiting in mon...%t", $time);
      `vmm_verbose(this.log, msg);
    end // else
    
  end : mon_push_loop
endtask : mon_push
  

  





