// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: Fifo_xactn
// PURPOSE: Represents the transaction to be sent to the channel by the transaction generator

class Fifo_xactn extends vmm_data;
  import fifo_pkg::*;
  rand fifo_scen_t kind;
  rand word_t  data;	// data to push
  rand bit [7:0] idle_cycles;
  rand bit [7:0] reset_cycles;
  time xactn_time;
  
  constraint cst_data {
      data  < 1024;
  }
 constraint cst_idle {
   idle_cycles inside {[1:3]}; 
  }  
 constraint cst_reset {
   reset_cycles inside {[1:10]}; 
  }  

  constraint cst_xact_kind {
    kind dist {
      PUSH := 400,
      POP := 300,
      PUSH_POP :=200,
      IDLE := 30, 
      RESET := 10
    };
  } // cst_xact_kind

static vmm_log log = new("fifo_push", "");

  function new();
    super.new(this.log);
  endfunction : new

  extern virtual function string psdisplay(string prefix = "");
  extern virtual function  vmm_data copy(vmm_data to=null);
  extern virtual function Fifo_xactn allocate();
  extern virtual function bit compare(vmm_data   to,
                               output string diff,
                               input int kind = -1		  
                               );
endclass:Fifo_xactn

function string Fifo_xactn::psdisplay(string prefix);
    $sformat(psdisplay,
             "%s #%0d.%0d.%0d Fifo Xaction %s ",
             prefix, this.scenario_id, this.stream_id, this.data_id,
	     this.kind.name());

    if (this.kind == IDLE) 
      $sformat(psdisplay, "%s Cycles %0d ", psdisplay, this.idle_cycles);
    if (this.kind == RESET) 
      $sformat(psdisplay, "%s Cycles %0d ", psdisplay, this.reset_cycles);
  
  endfunction : psdisplay

function  vmm_data Fifo_xactn::copy(vmm_data to);
  Fifo_xactn cpy;

  if (to !=null) begin 
    if (!$cast(cpy, to)) begin
      `vmm_fatal(log, "Attempting to copy a non fifo_xactn instance");
	  return;
	end
  end else cpy =new;
  super.copy_data(cpy);
  cpy.kind = this.kind;
  cpy.data = this.data;
  cpy.idle_cycles = this.idle_cycles;
  copy = cpy;
endfunction : copy 
  
function Fifo_xactn Fifo_xactn::allocate();
    Fifo_xactn fifo_xactn_local=new();
    allocate = fifo_xactn_local;
endfunction : allocate 

function bit Fifo_xactn::compare(vmm_data   to,
                               output string diff,
                               input int kind
                               );
begin
   Fifo_xactn cmp;
   string tmp_str;
 
   compare = 1; // Assume success by default.
   diff    = "";
 
   // Cast assign the vmm_data handle to an Fifo_xactn handle
   if (!$cast(cmp, to)) begin
      `vmm_fatal(this.log, "Attempting to compare to a non Fifo_xactn instance");
      compare = 0;
      diff = "Cannot compare non Fifo_xactn to Fifo_xactn";
      return;
   end
 
   // Compare the individual data members, and set compare to 0 and
   // the "diff" text string to "xxx field mismatched" on failure.
 
   if (this.data !== cmp.data) begin compare = 0; diff = {diff, "Data Mismatch "}; end
   if (compare == 0)begin
     diff = {"Fifo_xactn objects are not identical. Mismatched field(s) is(are):\n", diff};
   end
end
endfunction

    
      // This macro declares new derived class named:
      // fifo_xactn_channel from vmm_channel
`vmm_channel (Fifo_xactn)




