// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: s2p_wo_shift
// PURPOSE:  DUT for the s2p

module ser2par(par_data,par_data_valid,ser_eop,ser_data,ser_sop,pkt_abort,clk,rst_n);
  parameter           SIZE = 8;

  output [SIZE-1:0]    par_data;
  output 	       par_data_valid;
    
  input 	       ser_data,
                       ser_sop,
                       ser_eop,
		       pkt_abort,
		       clk,
		       rst_n;
	
  reg [SIZE-1:0]       par_data;
  reg 		       par_data_valid;
  reg [1:0] 	       state,state_nxt;
  reg [SIZE-1:0]       par_data_temp = 8'h00;
  reg 		       par_data_valid_temp = 1'b0;
   
  parameter 	       IDLE = 2'b00;
  parameter 	       P_DATA = 2'b01;
  parameter 	       P_END = 2'b10;

  reg [3:0] 	       count;
  reg 		       count_en = 1'b0;
   

  always @ (posedge clk or negedge rst_n )
  begin
    if( !rst_n )
    begin
      count <= 'b0;
    end
    else 
    begin
      par_data_temp <={ser_data,par_data_temp[SIZE-1:1]};
      if(count_en)
      begin
	count <= count +1;
      end
    end // else: !if( !rst_n )
  end // always @ (posedge clk or negedge rst_n )

   always @ (posedge clk)  
  begin
    if (! rst_n)
    begin
      par_data <= 8'h00;	
      par_data_valid <= 0;
      state <= IDLE;
    end
    else
    begin
      par_data <= par_data_temp;	
      par_data_valid <= par_data_valid_temp;
      state <= state_nxt;
    end
  end // always @ (posedge clk)
   
  
  always @ (*)
  begin
    case(state)
      IDLE: 
	begin
	  if(ser_sop && !pkt_abort)
	  begin
	    count_en = 1'b1;
	    par_data_valid_temp = 1'b0;
	    state_nxt = P_DATA;
	  end
	  else
	  begin
	    count_en = 1'b0;
	    count = 'b0;
	    par_data_temp = 8'h00;
	    par_data_valid_temp = 1'b0;
	    state_nxt = IDLE;
	  end
	end // case: IDLE
      
      P_DATA: 
	begin
	  if(pkt_abort)
	  begin
	    count_en = 1'b0;
	    count = 'b0;
	    par_data_temp = 8'h00;
	    par_data_valid_temp = 1'b0;
	    state_nxt = IDLE;
	  end
	  else 
	  begin
	    if((!ser_eop) && (count < SIZE))
	    begin
	      count_en = 1'b1;
	      par_data_valid_temp = 1'b0;
	      state_nxt = P_DATA;
	    end
	    else
	      if(ser_eop)
	      begin
		if(ser_sop && count == 7)
		begin
		  par_data_valid_temp = 1'b1;
		  count = 'b0;
		  count_en = 1'b0;
		  state_nxt = P_END;
		end
		else
		  begin
		    par_data_valid_temp = 1'b0;
		    count = 'b0;
		    count_en = 1'b0;
		    state_nxt = P_END;
		  end
	      end // if (ser_eop)
	      else
	      begin
		count = 'b0;
		count_en = 1'b1;
		par_data_valid_temp = 1'b1;
		state_nxt = P_DATA;
	      end // else: !if(ser_eop)
	    end // else: !if((!ser_eop) && (count < (SIZE )))
	end // case: P_DATA
      
	  
      P_END: 
	begin
	  if(ser_sop && !pkt_abort)
	  begin
	    count_en = 1'b1;
	    state_nxt = P_DATA;
	  end
	  else
	  begin
	    count_en = 1'b0;
	    count = 'b0;
	    par_data_valid_temp = 1'b0;
	    state_nxt = IDLE;
	  end
	end // case: P_END
    endcase // case(state)
  end
endmodule

