// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: s2p_scen_env
// PURPOSE: Creates the verification environment

`include "s2p_log_fmt.sv"
`include "s2p_xactn.sv"
`include "s2p_cmd_xactor.sv"
`include "s2p_scen_gen.sv"
`include "s2p_scenario_defn.sv"

class S2p_env extends vmm_env;
  S2p_cmd_xactor s2p_cmd_xactor_0;
  S2p_xactn_channel s2p_channel;
  
   
  Test_cfg test_cfg_0;
  S2p_xactn_scenario_gen s2p_sc_gen_0;
  S2p_log_fmt log_fmt_cntl;
  vmm_log log;
  
  function new();
    $timeformat(-9,2, " ns");
    this.log_fmt_cntl = new();
    this.log = new("S2P Env Logger", "0");
    this.log.set_format(this.log_fmt_cntl);
  endfunction : new

  extern function void gen_cfg();	
  extern function void build();
  extern task reset_dut();
  extern task  start();
  extern task wait_for_end();
endclass : S2p_env

function void S2p_env::gen_cfg();
    super.gen_cfg();
    this.test_cfg_0 = new;
    this.test_cfg_0.randomize();
  endfunction : gen_cfg
  
  function void S2p_env::build();
    string msg;
    super.build();
    this.s2p_channel = new("s2p_chan","0");
    this.s2p_cmd_xactor_0 = new("S2P_XACTOR",
                        0,
                        s2p_tb.s2p_if_0,
                        s2p_channel
                        );
    this.s2p_sc_gen_0 = new ("S2p Xaction generator", 0);
   
    this.s2p_sc_gen_0.stop_after_n_insts = this.test_cfg_0.no_of_xactions;
   $sformat(msg, "Sim shall run for no_of_xactions %0d ", 
	    this.s2p_sc_gen_0.stop_after_n_insts);
    `vmm_note(this.log, msg);
    this.s2p_sc_gen_0.out_chan = this.s2p_channel;
endfunction : build

 task S2p_env::reset_dut();
  super.reset_dut();
  #30 s2p_tb.rst_n <= 1'b0;
  s2p_tb.s2p_if_0.ser_cb.ser_sop <= 1'b0;
  s2p_tb.s2p_if_0.ser_cb.ser_eop <= 1'b0;
  s2p_tb.s2p_if_0.ser_cb.ser_abort <= 1'b0;
  s2p_tb.s2p_if_0.ser_cb.ser_data <= 1'b0;
  
    repeat (10) @(s2p_tb.s2p_if_0.ser_cb);
    #300 s2p_tb.rst_n  <= 1'b1;
    repeat (10) @(s2p_tb.s2p_if_0.ser_cb);
  endtask : reset_dut

  task S2p_env::start();
    super.start();
    this.s2p_sc_gen_0.start_xactor();
    this.s2p_cmd_xactor_0.start_xactor();
 endtask : start
  
  task S2p_env::wait_for_end();
    super.wait_for_end();
    fork : f_gen
      this.s2p_sc_gen_0.notify.wait_for(S2p_xactn_scenario_gen::DONE);
    join : f_gen
  `vmm_note(this.log, "Reached EOT!");
  
  endtask : wait_for_end