// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: s2p_xactn
// PURPOSE: Transaction sent by generator

class S2p_xactn extends vmm_data;

  rand logic [31:0] pkt_pld;
  rand logic err_pkt;
  rand bit [7:0] pkt_length;
  
  time xactn_time;
  
  constraint cst_no_err_pkt {
			     //err_pkt == 0;
			     } // cst_no_err_pkt

  static vmm_log log = new("s2p_xactn", "");

  function new();
    super.new(this.log);
  endfunction : new

  extern virtual function string psdisplay(string prefix = "");
  extern virtual function  vmm_data copy(vmm_data cpy = null);
  extern virtual function vmm_data allocate();
  extern virtual function bit compare(vmm_data   to,
                               output string diff,
                               input int kind = -1		  
                               );
endclass:S2p_xactn

function string S2p_xactn::psdisplay(string prefix);
  $sformat(psdisplay,
           "%s #%0d.%0d.%0d s2p %s Pkt, length %0d, data %0h",
           prefix, this.stream_id, this.scenario_id, this.data_id,
	   this.err_pkt ? "ERROR" : "GOOD",
	   this.pkt_length, this.pkt_pld);
endfunction : psdisplay
  

function  vmm_data S2p_xactn::copy(vmm_data cpy);
S2p_xactn local_xactn;

  if (cpy == null) begin
   local_xactn  = new;
  end
  else if (!$cast(local_xactn, cpy)) begin
    `vmm_fatal(log, "Attempting to copy a non s2p_xactn instance");
    copy = null;
    return;
  end
super.copy_data(local_xactn);

local_xactn.pkt_pld = this.pkt_pld;
local_xactn.err_pkt = this.err_pkt;
local_xactn.pkt_length = this.pkt_length;
return(local_xactn);
endfunction : copy
  
function vmm_data S2p_xactn::allocate();
    S2p_xactn s2p_xactn_local=new();
allocate = s2p_xactn_local;
endfunction : allocate 

function bit S2p_xactn::compare(vmm_data   to,
                               output string diff,
                               input int kind
                               );
begin
   S2p_xactn cmp;
   string tmp_str;
 
   compare = 1; // Assume success by default.
   diff    = "";
 
   // Cast assign the vmm_data handle to an S2p_xactn handle
   if (!$cast(cmp, to)) begin
      `vmm_fatal(this.log, "Attempting to compare to a non S2p_xactn instance");
      compare = 0;
      diff = "Cannot compare non S2p_xactn to S2p_xactn";
      return;
   end
 
   // Compare the individual data members, and set compare to 0 and
   // the "diff" text string to "xxx field mismatched" on failure.
 
   if (this.pkt_pld !== cmp.pkt_pld && this.pkt_length !== cmp.pkt_length)
   begin compare = 0; diff = {diff, "Payload and length  Mismatch "};
   end
if (compare == 0)
begin
  diff = {"S2p_xactn objects are not identical. Mismatched field(s) is(are):\n", diff};
end
end
endfunction
    
`vmm_channel (S2p_xactn)

