// VhdlCohen Publishing, copyright 2006 
// wwww.systemverilog.us  ben@systemverilog.us
// Code for use with the book 
// "A pragmatic Approach to Adopting VMM" ISBN 0-9705394-0-5
// NAME: s2p_env
// PURPOSE: Creates the verification environment

`include "s2p_log_fmt.sv"
`include "s2p_xactn.sv"
`include "s2p_cmd_xactor.sv"
`include "s2p_atomic_gen.sv"
`include "s2p_asink.sv"
class S2p_env extends vmm_env;
  S2p_cmd_xactor s2p_cmd_xactor_0;
  S2p_xactn_channel s2p_channel;
  S2p_xactn_channel s2p_src_channel[];
  S2p_xactn my_dummy_xactn;
  S2p_xactn_atomic_gen s2p_atom_gen_array [];
  Test_cfg test_cfg_0;
  S2p_log_fmt log_fmt_cntl;
  vmm_log log;
  S2p_asink s2p_asink;
  
  
  vmm_scheduler s2p_scheduler;
  
  function new();
    $timeformat(-9,2, " ns");
    this.log_fmt_cntl = new();
    this.log = new("S2P Env Logger", "0");
    this.log.set_format(this.log_fmt_cntl);
  endfunction : new

  extern function void gen_cfg();
  extern function void build();
  extern task reset_dut();
  extern task start();
  extern task wait_for_end();
  
endclass : S2p_env

  function void S2p_env::gen_cfg();
    super.gen_cfg();
    this.test_cfg_0 = new;
    this.test_cfg_0.randomize();
  endfunction:gen_cfg


  function void S2p_env::build();
    string msg;
    int iter;
    super.build();
    this.s2p_channel = new("s2p_chan","0");
   
    this.s2p_src_channel = new[8];
    this.s2p_atom_gen_array = new[8];
    foreach (s2p_src_channel[id]) begin: fe_loop_2
      this.s2p_src_channel[id] = new("s2p_chan[id]","0");
    end: fe_loop_2
  
    foreach (s2p_atom_gen_array [id] ) begin : fe_loop_0
     s2p_atom_gen_array[id] = new ("Gen", iter, 
				  s2p_src_channel[id]);
     s2p_atom_gen_array[id].stop_after_n_insts = this.test_cfg_0.no_of_xactions;
     iter++;
    
    end : fe_loop_0
  

    s2p_scheduler = new ("S2P Scheduler", "0", s2p_channel);
    foreach (s2p_src_channel[id]) begin : fe_loop_1
    s2p_scheduler.new_source(s2p_src_channel[id]);
    
    end : fe_loop_1
  
    `vmm_note(this.log, msg);

    this.s2p_asink = new("Artificial Sink", 0,s2p_channel);
      
    endfunction : build
  
  task S2p_env::reset_dut();
  super.reset_dut();
  #30 s2p_tb.rst_n <= 1'b0;
  s2p_tb.s2p_if_0.ser_cb.ser_sop <= 1'b0;
  s2p_tb.s2p_if_0.ser_cb.ser_eop <= 1'b0;
  s2p_tb.s2p_if_0.ser_cb.ser_abort <= 1'b0;
  s2p_tb.s2p_if_0.ser_cb.ser_data <= 1'b0;
  
    repeat (10) @(s2p_tb.s2p_if_0.ser_cb);
    #300 s2p_tb.rst_n  <= 1'b1;
    repeat (10) @(s2p_tb.s2p_if_0.ser_cb);
  endtask : reset_dut

  task S2p_env::start();
    super.start();
    foreach (s2p_atom_gen_array [id] ) begin : fe_loop_1
      this.s2p_atom_gen_array[id].start_xactor();
    end : fe_loop_1

  
    this.s2p_scheduler.start_xactor();
    this.s2p_asink.start_xactor();
    
 endtask : start
  
  task S2p_env::wait_for_end();
    super.wait_for_end();
    foreach (s2p_atom_gen_array [id] )begin : fe_l_4
    fork : f_gen
      this.s2p_atom_gen_array[id].notify.wait_for(S2p_xactn_atomic_gen::DONE);
    join : f_gen
      end : fe_l_4
#1000;

  `vmm_note(this.log, "Reached EOT!");
  
  endtask : wait_for_end
